#!/bin/sh

REV=r180267

for dir in src include lib; do
  rsync -a ../libcxx-trunk/$dir .
done

# When not using a versioning namespace, take from libstdc++.
rm -f include/exception
rm -f include/new
rm -f include/typeinfo

rm -f src/exception.cpp
rm -f src/new.cpp
rm -f src/typeinfo.cpp

patch include/locale <<EOP
--- locale.orig	2012-12-22 02:35:58.000000000 +0000
+++ locale	2012-12-24 23:48:34.951419970 +0000
@@ -202,6 +202,11 @@
 #pragma GCC system_header
 #endif
 
+namespace std {
+void
+__throw_bad_alloc(void);
+}
+
 _LIBCPP_BEGIN_NAMESPACE_STD
 
 #if __APPLE__ || __FreeBSD__
EOP

patch include/stdexcept <<EOP
--- stdexcept.orig	2012-12-25 01:25:56.417964000 +0000
+++ stdexcept	2012-12-25 01:26:14.247090000 +0000
@@ -50,8 +50,7 @@
 #pragma GCC system_header
 #endif
 
-namespace std  // purposefully not using versioning namespace
-{
+_LIBCPP_BEGIN_NAMESPACE_STD
 
 class _LIBCPP_EXCEPTION_ABI logic_error
     : public exception
@@ -157,6 +156,6 @@
     virtual ~underflow_error() _NOEXCEPT;
 };
 
-}  // std
+_LIBCPP_END_NAMESPACE_STD
 
 #endif  // _LIBCPP_STDEXCEPT
EOP

patch src/stdexcept.cpp <<EOP
--- stdexcept.cpp.orig	2012-12-25 01:28:25.236635000 +0000
+++ stdexcept.cpp	2012-12-25 01:29:31.897157000 +0000
@@ -93,8 +93,7 @@
 
 #pragma GCC visibility pop
 
-namespace std  // purposefully not using versioning namespace
-{
+_LIBCPP_BEGIN_NAMESPACE_STD
 
 logic_error::logic_error(const string& msg)
 {
@@ -193,4 +192,4 @@
 
 #endif
 
-}  // std
+_LIBCPP_END_NAMESPACE_STD
EOP

patch src/chrono.cpp <<EOP
--- chrono.cpp.orig	2012-12-22 02:35:58.000000000 +0000
+++ chrono.cpp	2013-03-01 06:04:29.617692000 +0000
@@ -121,8 +121,14 @@
 steady_clock::now() _NOEXCEPT
 {
     struct timespec tp;
+#ifdef __bg__ // The IBM Blue Gene compute nodes (on both P and Q) do not
+              // support CLOCK_MONOTONIC.
+    if (0 != clock_gettime(CLOCK_REALTIME, &tp))
+        __throw_system_error(errno, "clock_gettime(CLOCK_REALTIME) failed");
+#else
     if (0 != clock_gettime(CLOCK_MONOTONIC, &tp))
         __throw_system_error(errno, "clock_gettime(CLOCK_MONOTONIC) failed");
+#endif
     return time_point(seconds(tp.tv_sec) + nanoseconds(tp.tv_nsec));
 }
 #endif  // __APPLE__
EOP

BGCLANG_STATIC_LINKING=no
export BGCLANG_STATIC_LINKING

CC=/home/projects/llvm/bin/bgclang
CXX='/home/projects/llvm/bin/bgclang++ -D__GLIBCXX__'
export CC
export CXX
rm -f lib/libc++.so
rm -f lib/libc++.so.1
(cd lib && sh buildit && ln -s libc++.so.1.0 libc++.so.1 && ln -s libc++.so.1 libc++.so)

BGCLANG_STATIC_LINKING=yes
export BGCLANG_STATIC_LINKING

CC=true
export CC
sed -i 's/^#libtool -static -o/ar cr/g' lib/buildit

(cd lib && sh buildit)

mkdir -p /home/projects/llvm/libc++.$REV
mkdir -p /home/projects/llvm/libc++.$REV/include
mkdir -p /home/projects/llvm/libc++.$REV/lib

cp -r include/* /home/projects/llvm/libc++.$REV/include/
find /home/projects/llvm/libc++.$REV/include -name .svn | xargs rm -rf

cp -d lib/libc++.so* /home/projects/llvm/libc++.$REV/lib/
cp lib/libc++.a /home/projects/llvm/libc++.$REV/lib/

